from typing import Any, override
from pathlib import Path
from marko import Markdown
from marko.inline import Link, Image
from marko.html_renderer import HTMLRenderer
from marko.parser import Parser

from zona.models import Item


class ZonaRenderer(HTMLRenderer):
    def __init__(
        self,
        resolve: bool = False,
        source: Path | None = None,
        item_map: dict[Path, Item] | None = None,
    ):
        super().__init__()
        self.resolve: bool = resolve
        if self.resolve:
            assert source is not None
            assert item_map is not None
            self.source: Path = source.resolve()
            self.map: dict[Path, Item] = item_map

    # TODO: resolve relative links and replace with url?
    @override
    def render_link(self, element: Link):
        href = element.dest
        assert isinstance(href, str)
        if self.resolve:
            cur = Path(href)
            # TODO: fix relative path issue
            if not href.startswith(("http", "https")) and cur.is_relative_to(
                self.source
            ):
                href = self.map[self.source].url
        body: Any = self.render_children(element)
        return f'<a href="{href}" target="_blank">{body}</a>'

    @override
    def render_image(self, element: Image):
        # get alt text from children
        text = self.render_children(element)
        title = element.title or ""
        caption = f"<small>{text}</small>" if text else ""
        return (
            f'<div class="image-container">\n'
            # TODO: convert to plaintext and add as alt attribute
            f'<img src="{element.dest}" title="{title}">\n'
            f"{caption}</div>"
        )


def md_to_html(
    content: str,
    resolve_links: bool = False,
    source: Path | None = None,
    item_map: dict[Path, Item] | None = None,
) -> str:
    parser = Parser()
    ast = parser.parse(content)
    renderer = ZonaRenderer(resolve_links, source, item_map)
    return renderer.render(ast)
